'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var access_control_authorization_1 = require("./access-control.authorization");
var Q = require('q');
var _ = require('lodash');
function crudAccessControlWithOrgRolesFactory(resource, grants) {
    if (!resource || !grants)
        throw Error('missing parameters');
    var thisGrants = _.cloneDeep(grants);
    thisGrants.forEach(function (grant) { grant.resource = grant.resource || resource; });
    logger_1.default.warn('--------------####');
    logger_1.default.warn(thisGrants[0].resource);
    return Q.when(thisGrants)
        .then(instantiateCrudAccessControl(resource));
    /*
    return queryAllOrgs()
    .then(expandGrantsListRules(thisGrants))
    .then(instantiateCrudAccessControl(resource))
    */
}
exports.crudAccessControlWithOrgRolesFactory = crudAccessControlWithOrgRolesFactory;
function queryAllOrgs() {
    logger_1.default.trace('listing orgs');
    return Q.when([{
            code: 'vert'
        }, {
            code: 'rhases'
        }, {
            code: 'tass'
        }]);
}
function expandGrantsListRules(grants) {
    return function (organizations) {
        logger_1.default.trace('expanding grants');
        return grants.reduce(function (acc, grant) {
            if (_.startsWith(grant.role, '$organization')) {
                logger_1.default.trace('grant to be expanded');
                var orgRole = grant.role.slice('$organizationa'.length);
                var grunts = organizations.map(function (organization) {
                    var newGrant = _.clone(grant);
                    newGrant.role = organization.code + ":" + orgRole;
                    logger_1.default.trace("grants added with role " + newGrant.role);
                    return newGrant;
                });
                acc = _.concat(acc, grunts);
            }
            else {
                logger_1.default.trace("grants added as is '" + grant.role + "'");
                acc.push(grant);
            }
            return acc;
        }, []);
    };
}
function instantiateCrudAccessControl(resource) {
    return function (grants) {
        logger_1.default.trace("instantiate CrudAccessControl '" + resource + "' and '" + grants.length + " grants'");
        return new access_control_authorization_1.CrudAccessControl(resource, grants);
    };
}
