'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var Q = require('q');
var _ = require('lodash');
var functions_utils_1 = require("./functions.utils");
function setBasicQueries(schema) {
    schema.query.byUser = function (user) {
        if (user)
            this.where("userId").eq(user._id);
        return this;
    };
    return schema;
}
exports.setBasicQueries = setBasicQueries;
//export var TicketSchema;
function createQueryExecutor(model, _queryBuilder) {
    var self = this;
    return function (__queryBuilder) {
        //query builder func could be setted as param internal, external or both.
        var queryBuilder = functions_utils_1.concatFunctions(_queryBuilder, __queryBuilder);
        var executor = {};
        executor.execCount = function () {
            return self.execCount(model, queryBuilder);
        };
        executor.execFind = function (sortBy, currentPage, perPage, populate) {
            return self.execFind(model, queryBuilder, sortBy, currentPage, perPage, populate);
        };
        logger_1.default.trace('query executor created');
        return executor;
    };
}
exports.createQueryExecutor = createQueryExecutor;
function execFindAndCount(query, res) {
    return function (queryExecutor) {
        logger_1.default.trace("executing find and count " + queryExecutor);
        var sortBy = query.sort;
        var currentPage = query.page || 1;
        var perPage = query.per_page || 200;
        var populate = query.populate || '';
        return queryExecutor
            .execCount()
            .then(function (totalCount) {
            res.set('X-Total-Count', String(totalCount));
            return queryExecutor.execFind(sortBy, currentPage, perPage, populate);
        });
    };
}
exports.execFindAndCount = execFindAndCount;
//Query utils
function execCount(model, queryBuilder) {
    logger_1.default.info(queryBuilder, model.collection.collectionName + ".execCount()");
    logger_1.default.trace(typeof queryBuilder);
    return queryBuilder(model.count(null)).exec();
}
exports.execCount = execCount;
function execFind(model, queryBuilder, sortBy, currentPage, perPage, populate) {
    logger_1.default.info(queryBuilder, model.collection.collectionName + ".execFind()");
    var query = queryBuilder(model.find());
    if (sortBy)
        query.sort(sortBy);
    if (populate)
        query.populate(getPopulateObject(populate));
    if (currentPage > 0 && perPage > 0)
        query
            .skip((currentPage - 1) * perPage)
            .limit(perPage);
    return query.exec();
}
exports.execFind = execFind;
function execQuery(dQuery) {
    return dQuery.exec();
}
exports.execQuery = execQuery;
function createFindByIdQuery(model, id) {
    return function () {
        return model.findById(id);
    };
}
exports.createFindByIdQuery = createFindByIdQuery;
function execFindByIdWithQueryBuilder(model, id) {
    return function (queryBuilder) {
        return queryBuilder(model.findById(id))
            .exec();
    };
}
exports.execFindByIdWithQueryBuilder = execFindByIdWithQueryBuilder;
function restrictByOwner(ownerTypes, userId, organizationCode) {
    var restrictions = [];
    if (ownerTypes.indexOf('user') >= 0) {
        restrictions.push({ "owner.userId": userId });
    }
    if (ownerTypes.indexOf('organization') >= 0) {
        restrictions.push({ "owner.organizationCode": organizationCode });
    }
    return function (query) {
        if (restrictions.length == 0) {
            return query;
        }
        if (restrictions.length == 1) {
            return query.where(restrictions[0]);
        }
        else {
            return query.or(restrictions);
        }
    };
}
exports.restrictByOwner = restrictByOwner;
function getPopulateObject(populatePathParam) {
    var completePaths = populatePathParam.split(',');
    var populates = [];
    completePaths.forEach(function (completePath) {
        var paths = completePath.split('.');
        paths.reduce(function (fieldsToPopulate, path) {
            var p = _.find(fieldsToPopulate, { path: path });
            if (!p) {
                p = { path: path, populate: [] };
                fieldsToPopulate.push(p);
            }
            return p.populate;
        }, populates);
    });
    return populates;
}
// deprecated
function restrictByUserOwner(user) {
    return function (query) {
        return query.where("owner.userId").equals(user._id);
    };
}
exports.restrictByUserOwner = restrictByUserOwner;
// deprecated
function restrictByOrganizationOwner(user) {
    return function (query) {
        return query.where("owner.organizationCode").equals(user.organization.ref.code);
    };
}
exports.restrictByOrganizationOwner = restrictByOrganizationOwner;
