'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var Q = require('q');
var logger_1 = require("../logger");
var _ = require("lodash");
var createError = require('http-errors');
function findEntityById(model, id) {
    return model.findById(id)
        .exec();
}
exports.findEntityById = findEntityById;
function applyUpdate(updates) {
    return function (entity) {
        if (!entity)
            return;
        var updated = entity.merge(updates);
        return updated.save()
            .then(function (result) {
            return result;
        });
    };
}
exports.applyUpdate = applyUpdate;
function applyPatch(patches) {
    return function (entity) {
        if (!_.isArray(patches)) {
            logger_1.default.warn('trying to apply patches, but patches are not an array!');
            patches = [patches];
        }
        return patchAsPromised(entity, patches)
            .then(function (result) {
            logger_1.default.trace(result);
            return result;
        });
    };
}
exports.applyPatch = applyPatch;
function patchAsPromised(entity, patches) {
    var def = Q.defer();
    entity.patch(patches, function callback(err, result, number) {
        if (!err) {
            logger_1.default.trace("patched documents " + number);
            def.resolve(result.toObject());
        }
        else {
            logger_1.default.trace("patch rejected " + err);
            def.reject(createError(400, err));
        }
    });
    return def.promise;
}
function removeEntity() {
    return function (entity) {
        if (!entity)
            return;
        return entity.remove()
            .then(function (removed) {
            return removed;
        });
    };
}
exports.removeEntity = removeEntity;
function createEntity(model) {
    return function (entity) {
        logger_1.default.debug('creating entity...');
        logger_1.default.debug(entity);
        return model.create(entity);
    };
}
exports.createEntity = createEntity;
function setUserOwner(user) {
    return function (entity) {
        if (!entity.owner) {
            entity.owner = {};
        }
        logger_1.default.trace('setting user as owner');
        entity.owner.userId = user._id;
        return entity;
    };
}
exports.setUserOwner = setUserOwner;
function setOrganizationOwner(user) {
    return function (entity) {
        if (!entity.owner) {
            entity.owner = {};
        }
        entity.owner.organizationCode = user.organization.ref;
        return entity;
    };
}
exports.setOrganizationOwner = setOrganizationOwner;
function attributesFilter(filterHolder) {
    return function (entity) {
        return filterHolder.filter(entity);
    };
}
exports.attributesFilter = attributesFilter;
