"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var functions_utils_1 = require("./functions.utils");
describe('functions', function () {
    function double(num1) {
        return num1 + num1;
    }
    function quad(num1) {
        return 4 * num1;
    }
    function divideBy0(num1) {
        throw new Error('divide by 0');
    }
    it('should use first if defined', function () {
        var result = functions_utils_1.concatFunctions(double, undefined)(2);
        chai_1.expect(result).to.be.equal(4);
    });
    it('should use second if defined', function () {
        var result = functions_utils_1.concatFunctions(undefined, quad)(2);
        chai_1.expect(result).to.be.equal(8);
    });
    it('should use both if both are defined', function () {
        var result = functions_utils_1.concatFunctions(double, quad)(2);
        chai_1.expect(result).to.be.equal(16);
    });
    it('should be identity if none is defined', function () {
        var result = functions_utils_1.concatFunctions(undefined, undefined)(2);
        chai_1.expect(result).to.be.equal(2);
    });
    it('now should call then functions in chain', function () {
        var result = functions_utils_1.now(5)
            .then(double)
            .then(quad)
            .value();
        chai_1.expect(result).to.be.equal(40);
    });
    it('now should call err function', function () {
        var error;
        var result = functions_utils_1.now(5)
            .then(double)
            .then(divideBy0)
            .then(quad)
            .catch(function (err) { error = err; })
            .value();
        chai_1.expect(error.message).to.be.equal('divide by 0');
        chai_1.expect(result).to.be.undefined;
    });
});
