'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var Q = require('q');
var _ = require('lodash');
var createError = require('http-errors');
var logger_1 = require("../logger");
function assertGranted(permission) {
    var defer = Q.defer();
    if (!permission.granted) {
        defer.reject(createError(403, "not authorized for user with roles " + permission.allRoles));
    }
    else {
        logger_1.default.trace(permission);
        defer.resolve(permission);
    }
    return defer.promise;
}
exports.assertGranted = assertGranted;
function ifGranted(type, ownerType, grant, op) {
    if (grant.type && type === grant.type
        && (!ownerType || grant.ownerTypes.indexOf(ownerType) >= 0)) {
        return op;
    }
    else {
        logger_1.default.trace("not match for " + ownerType + ":" + type + " in this permission. Returning identity fnc");
        return function (value) { return value; }; //return nop
    }
}
exports.ifGranted = ifGranted;
function ifGrantedForOwn(grant, op) {
    return ifGranted('own', undefined, grant, op);
}
exports.ifGrantedForOwn = ifGrantedForOwn;
function ifGrantedForUser(grant, op) {
    return ifGranted('own', 'user', grant, op);
}
exports.ifGrantedForUser = ifGrantedForUser;
function ifGrantedForOrganization(grant, op) {
    return ifGranted('own', 'organization', grant, op);
}
exports.ifGrantedForOrganization = ifGrantedForOrganization;
function ifGrantedAny(grant, op) {
    return ifGranted('any', undefined, grant, op);
}
exports.ifGrantedAny = ifGrantedAny;
function ifDefined(func) {
    if (func) {
        return func;
    }
    else {
        return function (value) { return value; }; //return nop
    }
}
exports.ifDefined = ifDefined;
